(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('uuid'), require('os')) :
    typeof define === 'function' && define.amd ? define(['exports', 'uuid', 'os'], factory) :
    (factory((global['@yellicode/core'] = {}),global.uuid,global.os));
}(this, (function (exports,uuid,os) { 'use strict';

    /*
     * Copyright (c) 2018 Yellicode
     *
     * This Source Code Form is subject to the terms of the Mozilla Public
     * License, v. 2.0. If a copy of the MPL was not distributed with this
     * file, You can obtain one at http://mozilla.org/MPL/2.0/.
     */
    class StringUtility {
        static wordWrap(str, width) {
            const result = [];
            if (width < 1 || str == null || str.length <= width)
                return result;
            const len = str.length;
            let rangeMax = len - width; // we don't need to split after this position
            let rangeStart = 0;
            let subString;
            while (rangeStart < rangeMax) {
                let subLength;
                let rangeEnd = rangeStart + width;
                // First test for newlines in this range. If so, don't split on word break but on the newline.
                var ixNewLine = str.indexOf("\n", rangeStart);
                if (ixNewLine > -1 && ixNewLine > rangeStart && ixNewLine < rangeEnd) {
                    subLength = ixNewLine - rangeStart;
                    subString = str.substr(rangeStart, subLength);
                    rangeStart = rangeStart + subLength + 1;
                    result.push(subString);
                    continue;
                }
                // No newlines. Find the last wordbreak in the range.
                let ix = str.lastIndexOf(" ", rangeEnd); // find the last word break
                let rangeStartNext = 0;
                if (ix > -1 && ix != rangeStart - 1) {
                    subLength = ix - rangeStart;
                    if (subLength > 0) {
                        rangeStartNext = rangeStart + subLength + 1; // +1 to skip the whitespace
                    }
                    //else { // not needed anymore because of the maxPos check
                    //    sub = str.substr(pos);
                    //    newPos = len + 1;
                    //}
                }
                else {
                    // The range has no whitespace. 
                    subLength = width;
                    rangeStartNext = rangeStart + width;
                }
                subString = str.substr(rangeStart, subLength);
                rangeStart = rangeStartNext;
                result.push(subString.trim());
            }
            // Add the remainder
            if (rangeStart < len) {
                subString = str.substr(rangeStart);
                result.push(subString);
            }
            //  console.debug(result);
            return result;
        }
        /**
         * Makes the first character of the string uppercase.
         */
        static capitalize(str) {
            if (str == null || str.length === 0)
                return str;
            return str.charAt(0).toUpperCase() + str.slice(1);
        }
        /**
         * Makes the first character of the string lowercase.
         */
        static unCapitalize(str) {
            if (str == null || str.length === 0)
                return str;
            return str.charAt(0).toLowerCase() + str.slice(1);
        }
        /**
         * Returns a boolean value indicating if the string is upperCase.
         */
        static isUpperCase(str) {
            if (!str)
                return false;
            return (str === str.toUpperCase() && str !== str.toLowerCase());
        }
        /**
         * Returns a boolean value indicating if the string is lowerCase.
         */
        static isLowerCase(str) {
            if (!str)
                return false;
            return (str === str.toLowerCase() && str !== str.toUpperCase());
        }
    }

    /*
     * Copyright (c) 2018 Yellicode
     *
     * This Source Code Form is subject to the terms of the Mozilla Public
     * License, v. 2.0. If a copy of the MPL was not distributed with this
     * file, You can obtain one at http://mozilla.org/MPL/2.0/.
     */
    (function (LogLevel) {
        LogLevel[LogLevel["None"] = 0] = "None";
        LogLevel[LogLevel["Error"] = 1] = "Error";
        LogLevel[LogLevel["Warning"] = 2] = "Warning";
        LogLevel[LogLevel["Info"] = 3] = "Info";
        LogLevel[LogLevel["Verbose"] = 4] = "Verbose";
    })(exports.LogLevel || (exports.LogLevel = {}));
    (function (LogLevel) {
        function parse(str) {
            if (!str)
                return null;
            switch (str.toLowerCase()) {
                case 'none': return LogLevel.None;
                case 'error': return LogLevel.Error;
                case 'warning': return LogLevel.Warning;
                case 'info': return LogLevel.Info;
                case 'verbose': return LogLevel.Verbose;
                default: return null;
            }
        }
        LogLevel.parse = parse;
    })(exports.LogLevel || (exports.LogLevel = {}));
    class ConsoleLogger {
        constructor(console, level = exports.LogLevel.Info) {
            this.console = console;
            this.level = level;
        }
        verbose(message) {
            if (this.level < exports.LogLevel.Verbose)
                return;
            this.write("Verbose", message);
        }
        ;
        info(message) {
            if (this.level < exports.LogLevel.Info)
                return;
            this.write("Info", message);
        }
        ;
        warn(message) {
            if (this.level < exports.LogLevel.Warning)
                return;
            this.write("Warning", message);
        }
        ;
        error(message) {
            if (this.level < exports.LogLevel.Error)
                return;
            this.write("Error", message);
        }
        ;
        write(level, message) {
            var d = new Date();
            var n = d.toLocaleTimeString();
            this.console.log(`${n} - ${level}: ${message}`);
        }
    }

    /*
     * Copyright (c) 2018 Yellicode
     *
     * This Source Code Form is subject to the terms of the Mozilla Public
     * License, v. 2.0. If a copy of the MPL was not distributed with this
     * file, You can obtain one at http://mozilla.org/MPL/2.0/.
     */
    const HEX_LIST = '0123456789abcdef';
    const B64_LIST = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
    const STRING_LENGTH = 22; // Actually 24 characters for a base64 encoded Guid, but the padding "==" is removed.
    class UniqueId {
        static create() {
            const guid = uuid.v4();
            let encoded = UniqueId.guidToBase64(guid, UniqueId.isLittleEndian);
            // Modify for use in a URL https://en.wikipedia.org/wiki/Base64#URL_applications
            encoded = encoded.replace("/", "_").replace("+", "-");
            return encoded.substring(0, STRING_LENGTH); // remove the padding "=="
        }
        static guidToBase64(g, le) {
            // Credits: https://stackoverflow.com/a/9998010/9370196
            var s = g.replace(/[^0-9a-f]/ig, '').toLowerCase();
            if (s.length != 32)
                return '';
            if (le)
                s = s.slice(6, 8) + s.slice(4, 6) + s.slice(2, 4) + s.slice(0, 2) +
                    s.slice(10, 12) + s.slice(8, 10) +
                    s.slice(14, 16) + s.slice(12, 14) +
                    s.slice(16);
            s += '0';
            var a, p, q;
            var r = '';
            var i = 0;
            while (i < 33) {
                a = (HEX_LIST.indexOf(s.charAt(i++)) << 8) |
                    (HEX_LIST.indexOf(s.charAt(i++)) << 4) |
                    (HEX_LIST.indexOf(s.charAt(i++)));
                p = a >> 6;
                q = a & 63;
                r += B64_LIST.charAt(p) + B64_LIST.charAt(q);
            }
            r += '==';
            return r;
        }
    }
    UniqueId.isLittleEndian = os.endianness() === 'LE';

    /*
     * Copyright (c) 2018 Yellicode
     *
     * This Source Code Form is subject to the terms of the Mozilla Public
     * License, v. 2.0. If a copy of the MPL was not distributed with this
     * file, You can obtain one at http://mozilla.org/MPL/2.0/.
     */

    exports.StringUtility = StringUtility;
    exports.ConsoleLogger = ConsoleLogger;
    exports.UniqueId = UniqueId;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
